/*
 * Copyright 2011 GantzGulch, Inc.
 * 
 * This file is part of GantzHtmlTaglibs.
 *
 * GantzHtmlTaglibs is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GantzHtmlTagLibs is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with GantzHtmlTagLibs.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.gantzgulch.taglibs.html;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;

import com.gantzgulch.taglibs.Strings;

public class AbstractHtmlTag  extends AbstractTag {

	private static final long serialVersionUID = -4465428811941481286L;
	
	private final String elementName;
	
	public AbstractHtmlTag(String elementName) {
		this.elementName = elementName;
	}
	
	public void setClass(String className){
		setAttribute("class", className);
	}
	
	public void setDir(String dir){
		setAttribute("dir", dir);
	}
	
	public void setLang(String lang){
		setAttribute("lang", lang);
	}
	
	public void setStyle(String style){
		setAttribute("style", style);
	}
	
	public void setTitle(String title){
		setAttribute("title", title);
	}
	
	public void setOnclick(String onclick){
		setAttribute("onclick", onclick);
	}

	public void setOndblclick(String ondblclick){
		setAttribute("ondblclick", ondblclick);
	}

	public void setOnmousedown(String onmousedown){
		setAttribute("onmousedown", onmousedown);
	}
	
	public void setOnmousemove(String onmousemove){
		setAttribute("onmousemove", onmousemove);
	}
	
	public void setOnmouseout(String onmouseout){
		setAttribute("onmouseout", onmouseout);
	}
	
	public void setOnmouseover(String onmouseover){
		setAttribute("onmouseover", onmouseover);
	}
	
	public void setOnmouseup(String onmouseup){
		setAttribute("onmouseup", onmouseup);
	}
	
	public void setOnkeydown(String onkeydown){
		setAttribute("onkeydown", onkeydown);
	}
	
	public void setOnkeypress(String onkeypress){
		setAttribute("onkeypress", onkeypress);
	}
	
	public void setOnkeyup(String onkeyup){
		setAttribute("onkeyup", onkeyup);
	}
	
	protected void setAttribute(String name, String value){
		if( Strings.isNotBlank(value) ){
			setValue(name, value);
		}else{
			removeValue(name);
		}
	}
	
	protected String getAttribute(String name){
		return (String)getValue(name);
	}
	public int doEndTag() throws JspException {
		
		try {
			String bodyText = bodyContent != null ? bodyContent.getString() : null;
			pageContext.getOut().print( createElement(bodyText) );
		} catch (IOException e) {
			throw new JspTagException(e);
		}
		
		return EVAL_PAGE;
	}
	
	public String appendTimestamp(String url){
		return url.contains("?") ? url + "&ts=" + getTagTimestamp() : url + "?ts=" + getTagTimestamp();
	}
	
	public String createElement(String bodyText){
		
		StringBuilder builder = new StringBuilder();

		builder.append("<").append(elementName);

		List<String> sortedKeys = getSortedKeys();
		for(String key : sortedKeys){
			builder.append(" ").append(key).append("=\"").append(getValue(key)).append("\"");
		}

		if( bodyText != null ){
			builder.append(">").append(bodyText).append("</").append(elementName).append(">");
		}else{
			builder.append(" />");
		}

		
		return builder.toString();
	}

	private List<String> getSortedKeys(){
		List<String> sortedKeys = new ArrayList<String>();

		Enumeration<String> attributeKeyEnum = getValues();
		while( attributeKeyEnum.hasMoreElements() ){
			String key = attributeKeyEnum.nextElement();
			sortedKeys.add(key);
		}
		
		Collections.sort(sortedKeys);
		
		return sortedKeys;
	}
}
