/*
 * Copyright 2011 GantzGulch, Inc.
 * 
 * This file is part of GantzHtmlTaglibs.
 *
 * GantzHtmlTaglibs is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GantzHtmlTagLibs is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with GantzHtmlTagLibs.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.gantzgulch.taglibs.html;

import static org.hamcrest.Matchers.equalTo;
import static org.junit.Assert.assertThat;

import javax.servlet.jsp.JspException;

import org.junit.Before;
import org.junit.Test;

public class ImageTagTest {

	private ImageTag imageTag;
	private BodyTagRunner bodyTagRunner;
	
	@Before
	public void before() throws JspException {
		imageTag = new ImageTag();
		imageTag.setTagTimestamp("153563");
		bodyTagRunner = new BodyTagRunner(imageTag, "/app");
	}
	
	@Test
	public void generateTagWithOnlySrcAttributeEmptyBody() throws JspException {
		
		String expectedElement = "<img src=\"/app/images/test.png?ts=153563\" />";
		
		imageTag.setSrc("/images/test.png");
		
		bodyTagRunner.execute(null);
		
		assertThat(bodyTagRunner.getOutput(), equalTo(expectedElement));
	}

	@Test
	public void generateTagWithOnlySrcAttributeTextBody() throws JspException {
		
		String expectedElement = "<img src=\"/app/images/test.png?ts=153563\">content</img>";
		
		imageTag.setSrc("/images/test.png");
		
		bodyTagRunner.execute("content");
		
		assertThat(bodyTagRunner.getOutput(), equalTo(expectedElement));
	}

	@Test
	public void generateTagWithMultipleAttributesTextBody() throws JspException {
		
		String expectedElement = "<img alt=\"Test image\" src=\"/app/images/test.png?ts=153563\">content</img>";
		
		imageTag.setSrc("/images/test.png");
		imageTag.setAlt("Test image");
		
		bodyTagRunner.execute("content");
		
		assertThat(bodyTagRunner.getOutput(), equalTo(expectedElement));
	}
}
