/*
 * Copyright 2011 GantzGulch, Inc.
 * 
 * This file is part of GantzHtmlTaglibs.
 *
 * GantzHtmlTaglibs is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GantzHtmlTagLibs is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with GantzHtmlTagLibs.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.gantzgulch.taglibs.html;

import static org.hamcrest.Matchers.equalTo;
import static org.junit.Assert.assertThat;

import javax.servlet.jsp.JspException;

import org.junit.Before;
import org.junit.Test;

public class ScriptTagTest {

	private ScriptTag scriptTag;
	private BodyTagRunner bodyTagRunner;
	
	@Before
	public void before() throws JspException {
		scriptTag = new ScriptTag();
		scriptTag.setTagTimestamp("153563");
		bodyTagRunner = new BodyTagRunner(scriptTag, "/app");
	}
	
	@Test
	public void generateTagWithOnlySrcAttributeEmptyBody() throws JspException {
		
		String expectedElement = "<script src=\"/app/js/init.js?ts=153563\"> </script>";
		
		scriptTag.setSrc("/js/init.js");
		
		bodyTagRunner.execute(" ");
		
		assertThat(bodyTagRunner.getOutput(), equalTo(expectedElement));
	}

	@Test
	public void generateTagWithOnlySrcAttributeTextBody() throws JspException {
		
		String expectedElement = "<script src=\"/app/js/init.js?ts=153563\" type=\"text/javascript\"> </script>";
		
		scriptTag.setSrc("/js/init.js");
		scriptTag.setType("text/javascript");
		
		bodyTagRunner.execute(" ");
		
		assertThat(bodyTagRunner.getOutput(), equalTo(expectedElement));
	}

}
